@@ -34,6 +34,9 @@ extern "C" {
 #include <stdio.h>
 #include <dirent.h>
 
+#include <time.h>			// req for sleep
+#include <sys/time.h>		// req for getticks
+
 
 //#######################################################
 
@@ -83,7 +86,29 @@ char *		liqapp_getopt_str(char *optname,char *def);
 int 		liqapp_getopt_int(char *optname,int def);
 int 		liqapp_sleep(unsigned long millisec);
 
+
+
 int 		liqapp_formatnow(char *buffer,int buffersize,char *format);
+// 20090718_163211 lcuk : example of use:
+
+// char datestamp[20];
+// liqapp_formatnow(datestamp,sizeof(datestamp),"yyyymmdd_hhmmss");
+
+// ...
+
+// struct tm timebuf={0};
+// liqapp_datestamp_to_date(datestamp,&timebuf);
+// time_t w00t = mktime(&timebuf);
+// if(w00t==-1)
+// {
+//		// invalid
+// }
+
+
+
+int liqapp_datestamp_to_date(char *datestamp,struct tm *timebuf);		// convert a liqbase datestamp "yyyymmdd_hhmmss" into a tm struct
+
+
 char * 		liqapp_format_strftime(char *buffer,int buffersize,char *strftime_fmt);
 
 
