@@ -307,8 +307,8 @@ int liqcell_child_arrange_nooverlap(liqcell *self,liqcell *currentselection)
 	}
 
 
-	//int tilew =  liqcell_propgeti(self,"easytilew",0) * 0.9;
-	//int tileh =  liqcell_propgeti(self,"easytileh",0) * 0.9;
+	int tilew =  liqcell_propgeti(self,"liqcell_child_arrange_nooverlap_minimumw",0);
+	int tileh =  liqcell_propgeti(self,"liqcell_child_arrange_nooverlap_minimumh",0);
 
 	//liqapp_log("tile %i,%i",tilew,tileh);
 
@@ -319,23 +319,30 @@ int liqcell_child_arrange_nooverlap(liqcell *self,liqcell *currentselection)
 		{
 			if( (c->selected==0) && (c!=currentselection) )
 			{
-
-				c->x+=c->overlapx/2;
-				c->y+=c->overlapy/2;
+				if(c->overlapx || c->overlapy)
+				{
+					c->x+=c->overlapx;
+					c->y+=c->overlapy;
+					liqcell_setdirty(c,1); 
+				}
 
 				liqcell_forceinboundparent(c);
 
 			}
-/*
+
 			if(tilew && tileh)
 			{
-				if(c==currentselection)
+				if(c->selected || c==currentselection)
 				{
 					if((c->overlapx==0) && (c->overlapy==0))
 					{
 						// are free to make it a bit bigger
-						if( c->w < (self->w/2) ) {c->x-=2; c->w += 4;}
-						if( c->h < (self->h/2) ) {c->y-=2; c->h += 4;}
+						if( c->w < (self->w/2) ) {c->x-=2; c->w += 4;  liqcell_setdirty(c,1); }
+						if( c->h < (self->h/2) ) {c->y-=2; c->h += 4;  liqcell_setdirty(c,1); }
+						if( (c->selected==0) && (c!=currentselection) )
+						{
+							liqcell_forceinboundparent(c);
+						}
 					}
 				}
 				else
@@ -348,8 +355,12 @@ int liqcell_child_arrange_nooverlap(liqcell *self,liqcell *currentselection)
 					{
 						// are free to make it a bit smaller
 						// need to however find the smallest size
-						if( (c->w-1) > (tilew) ) {c->x++; c->w-=2;}
-						if( (c->h-1) > (tileh) ) {c->y++; c->h-=2;}
+						if( (c->w-1) > (tilew) ) {c->x++; c->w-=2;  liqcell_setdirty(c,1); }
+						if( (c->h-1) > (tileh) ) {c->y++; c->h-=2;  liqcell_setdirty(c,1); }
+						if( (c->selected==0) && (c!=currentselection) )
+						{
+							liqcell_forceinboundparent(c);
+						}
 					}
 				}
 
@@ -357,8 +368,6 @@ int liqcell_child_arrange_nooverlap(liqcell *self,liqcell *currentselection)
 
 			}
 
-*/
-				liqcell_forceinboundparent(c);
 		}
 		c=liqcell_getlinknext(c);
 	}
@@ -383,9 +392,6 @@ int liqcell_child_arrange_nooverlap(liqcell *self,liqcell *currentselection)
  * @return int finished 1 if at target, 0 if still in flight, -1 error
  * 
  */
-
-
-
 int liqcell_child_arrange_makegrid_internal(liqcell *self,int viscolcount,int visrowcount,int flymode)
 {
 
@@ -566,8 +572,6 @@ int liqcell_child_arrange_makegrid_fly(liqcell *self,int viscolcount,int visrowc
  * @return int Success or Failure
  * 
  */
-
-
 int liqcell_child_arrange_makegrid(liqcell *self,int viscolcount,int visrowcount)
 {
 	return liqcell_child_arrange_makegrid_internal(self,viscolcount,visrowcount,0);
@@ -621,7 +625,19 @@ int liqcell_child_arrange_easytile(liqcell *self)
 	//################################################ Determine the number of rows/cols
 	liqapp_log("liqcell_child_arrange_easytile weighing");
 	int ccols=1;
-	while(ccols<3 && ccols<answercount)ccols++;
+	// 20090624_010635 lcuk : changed from <3 to <5 to allow for more columns by default on busy ui's
+	if(answercount<=12)
+	{
+		while(ccols<3 && ccols<answercount)ccols++;
+	}
+	else
+	{
+		if(answercount<=25)
+			while(ccols<5 && ccols<answercount)ccols++;
+		else
+			while(ccols<7 && ccols<answercount)ccols++;
+		
+	}
 	int crows=answercount/ccols;
 	while(ccols*crows<answercount)crows++;
 
@@ -654,7 +670,11 @@ int liqcell_child_arrange_easytile(liqcell *self)
 }
 
 
-
+/**
+ * Arrange parent's child cells into a simple row
+ * @param self The liqcell to arrange the children of
+ * @return int Success or Failure
+ */
 int liqcell_child_arrange_easyrow(liqcell *self)
 {
 	liqcell *c;
@@ -703,8 +723,11 @@ int liqcell_child_arrange_easyrow(liqcell *self)
 	return 0;
 }
 
-
-
+/**
+ * Arrange parent's child cells into a simple column
+ * @param self The liqcell to arrange the children of
+ * @return int Success or Failure
+ */
 int liqcell_child_arrange_easycol(liqcell *self)
 {
 	liqcell *c;
