@@ -1,4 +1,3 @@
-
 /**
  * @file	liqcell_dllcache.c
  * @author  Gary Birkett
@@ -90,7 +89,7 @@ static int                 dllcache_used=0;
 
 
 static dllcacheitem *runstack[256];
-static int           runstack_size=256;
+//static int           runstack_size=256;
 static int           runstack_used=0;
 
 
@@ -405,6 +404,8 @@ liqcell *dllcache_runconstructorinner(char *classname)
 		}
 	}
 	
+	return NULL;
+	
 	
 
 }
