@@ -1,6 +1,10 @@
-#include <stdlib.h>
-#include <stdio.h>
+
+
+#define _GNU_SOURCE
 #include <string.h>
+#include <stdlib.h>
+
+#include <stdio.h>
 #include <X11/keysym.h>
 #include <stdarg.h>
 #include <unistd.h>
@@ -46,7 +50,7 @@ liqimage *easypaint_barcode_image = NULL;
 
 
 
-int thread_createwithpriority(pthread_t *tid,int threadpriority,void (*func)(void *),void *arg)
+int thread_createwithpriority(pthread_t *tid,int threadpriority,void *(*func)(void *),void *arg)
 {
 //pthread_t 		tid;
 pthread_attr_t 	tattr;
@@ -92,7 +96,8 @@ void *mainthread(void* mainthread_data)
 	
 do
 {
-	liqapp_sleep(10 + (rand() % 100));
+	//liqapp_sleep(10 + (rand() % 100));
+	liqapp_sleep(10 + (50));
 }
 while(mainthread_inprogress>1);
 
@@ -267,9 +272,36 @@ while(mainthread_inprogress>1);
 
 
 
+int liqcell_threadloadimage(liqcell *self)
+{
+				if(!easypaint_isloading_image)
+				{
+					//easypaint_isloading_image=liqimage_newfromfile("/usr/share/liqbase/libliqbase/media/sun.png",0,0,0);
+					easypaint_isloading_image=liqimage_newfromfile("/usr/share/liqbase/libliqbase/media/pleasewait.png",0,0,0);
+				}
+				if(easypaint_isloading_image)
+				{
+					liqcell_hold(self);
+					// only if we have a valid replacer can we do this trickery :)
+					liqcell_setimage(self, liqimage_hold(easypaint_isloading_image) );
+					// now we must start the thread off
+					pthread_t 		tid;
+					
+					//int tres=thread_createwithpriority(&tid,0,mainthread,self);
+					
+					int tres=pthread_create(&tid,NULL,mainthread,self);
+					
+					
+					if(tres)
+					{
+						liqapp_log("liqcell_easypaint: thread create fail %s :: %i",liqcell_getcaption(self),tres);
+						//liqcliprect_release(cr);
+						liqcell_release(self);
+						return;
+					}
+				}
 
-
-
+}
 
 
 
@@ -307,7 +339,7 @@ unsigned int decodecolor(char *source,unsigned char *ry,unsigned char *ru,unsign
 	char *s1=NULL;
 	char *s2=NULL;
 	char *s3=NULL;
-	char *s4=NULL;
+	//char *s4=NULL;  // used for alpha (not in place..)
 	if(strncmp(indat,"rgb(",4) == 0 )
 	{
 		indat+=4;
@@ -402,11 +434,11 @@ static int liqcell_kineticboiloff(liqcell *self)
 
 
 
-		if(px>0)px=0;
-		if(py>0)py=0;
+		if(px>0){px=0;  dx=self->kineticx; } // make sure kinetics are stopped
+		if(py>0){py=0;  dy=self->kineticy; } // make sure kinetics are stopped
 
-		if( (self->w > par->w) && ((px+self->w) < par->w) ) px = par->w - self->w;
-		if( (self->h > par->h) && ((py+self->h) < par->h) ) py = par->h - self->h;
+		if( (self->w > par->w) && ((px+self->w) < par->w) ){ px = par->w - self->w;  dx=self->kineticx; } // make sure kinetics are stopped
+		if( (self->h > par->h) && ((py+self->h) < par->h) ){ py = par->h - self->h;  dy=self->kineticy; } // make sure kinetics are stopped
 
 		//liqapp_log("kinetic pre motion... k=%2i,%2i  d=%i,%i    p=%i,%i s=%i,%i", self->kineticx , self->kineticy,   dx,dy,    px,py , self->x,self->y );
 
@@ -435,15 +467,14 @@ void liqcell_easypaint(liqcell *self,liqcliprect *crorig,    int x,int y,    int
 	//liqapp_log("#################################### liqcell easypaint (%i,%i) :: %s   xy(%i,%i)-wh(%i,%i)  cr==can.cr ? %i",self->w,self->h,self->name,x,y,w,h,crorig==liqcanvas_getcliprect());
 	
 	
-	
+
+// 20090517_012357 lcuk : turn logging on or off
+#ifdef __tz_easypaint_debug
 #define tzmax 25
 char 	tstr[tzmax][8]={{0}};
 unsigned long 	tz[tzmax]={0};
 long tzused=0;
-
-// 20090517_012357 lcuk : turn logging on or off
-#ifdef __tz_easypaint_debug
-#define __tz_one(code) { if(tzused<tzmax){  snprintf(tstr[tzused],32,"%s",(code));      tz[tzused++] = liqapp_GetTicks(); } }
+#define __tz_one(code) { if(tzused<tzmax){  snprintf(tstr[tzused],32,"%s",(code));      tz[tzused++] = liqapp_GetTicks(); } liqapp_log("%s $$ %s",self->name,code); }
 #else
 #define __tz_one(code) {  }
 #endif
@@ -455,7 +486,7 @@ __tz_one("start");
 
 
 
-char cy,cu,cv;
+//char cy,cu,cv;
 	if(w<1 || h<1)
 	{
 		//liqapp_log("size0 bail!");
@@ -572,6 +603,11 @@ __tz_one("showndone");
 		paintargs.graph = graph;
 		paintargs.runfast=0;
 		
+		paintargs.ox=x;
+		paintargs.oy=y;
+		paintargs.mx=w;
+		paintargs.my=h;
+		
 		liqcell_handlerrun(self,"paint",&paintargs);
 		
 		//vgraph_setwindow( graph,  NULL       );
@@ -646,8 +682,8 @@ __tz_one("backdone");
 				
 				if(!easypaint_isloading_image)
 				{
-					//easypaint_isloading_image=liqimage_newfromfile("media/sun.png",0,0,0);
-					easypaint_isloading_image=liqimage_newfromfile("/usr/share/liqbase/media/pleasewait.png",0,0,0);
+					//easypaint_isloading_image=liqimage_newfromfile("/usr/share/liqbase/libliqbase/media/sun.png",0,0,0);
+					easypaint_isloading_image=liqimage_newfromfile("/usr/share/liqbase/libliqbase/media/pleasewait.png",0,0,0);
 				}
 				if(easypaint_isloading_image)
 				{
@@ -664,10 +700,10 @@ __tz_one("backdone");
 					
 					if(tres)
 					{
-						liqapp_log("thread create fail %s :: %i",fn,tres);
+						liqapp_log("liqcell_easypaint: thread create fail %s :: %i",fn,tres);
 						liqcliprect_release(cr);
 						liqcell_release(self);
-						return 0;
+						return;
 					}
 				}
 			}
@@ -680,14 +716,161 @@ __tz_one("backdone");
 			
 		}
 	}
+	
+	
+	
+	
+	
+	
 __tz_one("imageprep");
 
 	if(self->image)
 	{
-		if(liqcell_propgeti(self,"lockaspect",0)==1)
-			liqcliprect_drawimagecolor(cr,self->image,x,y,w,h,1);
+		// 20090812_213106 lcuk : see if we should be floating this image
+		// 20090812_213118 lcuk : floating image is slightly larger than its default area and in constant slow motion
+		
+		char *fstr=liqcell_propgets(self,"imagefloat",NULL);
+		if(fstr && *fstr)
+		{
+		//	liqapp_log("imagefloat: start '%s' %i,%i",self->name,w,h);
+/*
+
+			//liqapp_log("imagefloat: start");
+			// when floating, the image is scaled 130%
+			int cx=0;					// cell dimensions
+			int cy=0;
+			int cw=liqcell_getw(self);
+			int ch=liqcell_geth(self);
+			
+			int tx=0;					// total dimensions
+			int ty=0;
+			int tw=cw*1.3;
+			int th=ch*1.3;
+			
+			int frw=tw-cw;
+			int frh=th-ch;
+			
+			float fax=0.03 * (float)(rand() % 100 - 50);	// acceleration is random
+			float fay=0.03 * (float)(rand() % 100 - 50);			
+			float fvx=0;									// velocity is some of the accel
+			float fvy=0;	
+			float fpx=0;									// position is the field
+			float fpy=0;
+			
+			//liqapp_log("imagefloat: reading");
+			int rs=sscanf(fstr,"%f,%f,%f,%f",&fvx,&fvy,&fpx,&fpy);
+			
+			//liqapp_log("imagefloat: read, got %i",rs);
+			
+			#define dt 0.2
+			#define maxv 10
+			
+			if(rs!=4){  fvx=0;  fvy=0;  fpx=0;  fpy=0; }
+			
+			//if(rs==4)
+			{
+				
+				
+				fvx += dt * fax;
+				fvy += dt * fay;
+				
+				if(fvx<-maxv){fvx=-maxv; } else{ if(fvx>maxv){fvx=maxv;}}
+				if(fvy<-maxv){fvy=-maxv; } else{ if(fvy>maxv){fvy=maxv;}}
+				
+				fpx += 0.1 * fvx;
+				fpy += 0.1 * fvy;
+				
+				if(fpx<0){ fpx=0;fvx=0;}else{ if(fpx>frw){fpx=frw;fvx=0;} }
+				if(fpy<0){ fpy=0;fvy=0;}else{ if(fpy>frh){fpy=frh;fvy=0;} }
+				
+				//liqapp_log("imagefloat: writing new");
+				
+				liqcell_propsets_printf(self,"imagefloat","%f,%f,%f,%f",fvx,fvy,fpx,fpy);
+				
+				
+				//liqapp_log("imagefloat: getting dims");
+				
+				int rw=w*1.3;
+				int rh=h*1.3;
+				
+				int xx=x- (fpx*w/cw);
+				int yy=y- (fpy*h/ch);
+				
+ 
+			}
+
+ */
+			
+			{
+
+
+				//################################################## lj variaton
+				unsigned long timer1 = liqapp_GetTicks();
+				
+				
+				float secs = ((float)timer1 / 1000.0);
+				
+				
+				
+				unsigned int rndofname = *(unsigned int *)&self->name;
+				
+				secs+=rndofname;
+
+
+
+		
+				#define PI 3.141592654
+				
+				float lrw=(w*0.3)/2;
+				float lrh=(h*0.3)/2;
+				
+				float lfx=2;
+				float lfy=3;
+				
+				float lalpha = 2 * PI * secs / 50;
+				float lx = lrw * cos(lfx * lalpha);
+				float ly = lrh * sin(lfy * lalpha);
+				
+				int rw=w*1.3;
+				int rh=h*1.3;
+
+				int xx=x-(lrw-lx);
+				int yy=y-(lrh-ly);
+				
+				
+				//http://jrgraphix.net/research/flash-lissajous-curves.php
+		
+		
+		
+
+				//liqapp_log("imagefloat: '%s' orig  %4i,%4i - %4i,%4i     into     %4i,%4i - %4i,%4i",self->name,x,y,w,h,  xx,yy,rw,rh );
+
+
+				if(liqcell_propgeti(self,"lockaspect",1)==1)
+					liqcliprect_drawimagecolor(cr,self->image,xx,yy,rw,rh,1);
+				else
+					liqcliprect_drawimagecolor(cr,self->image,xx,yy,rw,rh,0);
+					
+				//liqapp_log("imagefloat: making dirty");
+					
+				liqcell_setdirty(self,1);
+				
+				//liqapp_log("imagefloat: done");
+
+			}
+		}
 		else
-			liqcliprect_drawimagecolor(cr,self->image,x,y,w,h,0);
+		{
+
+				//liqapp_log("imagestd: '%s' draw  %4i,%4i - %4i,%4i    ",self->name,x,y,w,h );
+
+			// standard..
+			if(liqcell_propgeti(self,"lockaspect",1)==1)
+				liqcliprect_drawimagecolor(cr,self->image,x,y,w,h,1);
+			else
+				liqcliprect_drawimagecolor(cr,self->image,x,y,w,h,0);
+		}
+		
 	}
 
 __tz_one("imagedone");
@@ -798,7 +981,6 @@ __tz_one("fontprep");
 			{
 
 
-
 				char timebuf[128]="";
 				if( (self->classname) && (strcmp(self->classname,"time") ==0) )
 				{
@@ -811,6 +993,7 @@ __tz_one("fontprep");
 				}
 
 
+				int captionlen = strlen(caption);
 
 
 				int xx=x;
@@ -827,6 +1010,11 @@ __tz_one("fontprep");
 				int selstart = liqcell_propgeti(  self,"selstart",-1);
 				int sellength = liqcell_propgeti(  self,"sellength",0);
 				int cursorpos = liqcell_propgeti(  self,"cursorpos",-1);
+				
+				
+				if(selstart>captionlen)selstart=captionlen;
+				if(selstart+sellength>captionlen)sellength=captionlen-selstart;
+				if(cursorpos>captionlen)cursorpos=captionlen;
 
 
 
@@ -883,6 +1071,40 @@ __tz_one("fontprep");
 
 				if(selstart>=0)
 				{
+					if(fw>w)
+					{
+
+						// adjust offset of the text rendering to make sure cursor is on screen
+						char *sel=strndup(caption,cursorpos);
+						int tcx = liqfont_textwidth(self->font,sel);
+						if(tcx > w)
+						{
+							x-= (tcx - w);
+						
+						
+							char *more=strndup(caption+cursorpos,4);
+							int tex = liqfont_textwidth(self->font,more);
+							x-= tex;
+							x-= 16;
+							if(more)free(more);
+							
+							
+						}
+						if(sel)free(sel);
+						
+						
+
+						// pre   ss     curs     se      
+						// P1....P2......P3......P4.........
+						//char *p1=strndup(caption                   ,selstart                                );
+						//char *p2=strndup(caption+selstart          ,cursorpos-selstart                      );
+						//char *p3=strndup(caption+selstart          ,sellength                               );
+						//char *p4=strndup(caption+selstart+sellength,captionlen-(caption+selstart+sellength) );
+											
+					}
+						
+					
+					
 					// with selection
 					if(sellength==0)
 					{
@@ -894,6 +1116,9 @@ __tz_one("fontprep");
 						// draw upto selstart in normal color
 						// draw all of sellength
 						// draw the remainder
+						
+						
+						int tx=x;
 
 
 
@@ -911,7 +1136,7 @@ __tz_one("fontprep");
 						{
 							char *sel=strndup(alline,sellength);
 							int ttt = liqfont_textwidth(self->font,sel);
-							free(sel);
+							if(sel)free(sel);
 							liqcliprect_drawboxfillcolor(cr,x,y,ttt,fh,tcy,tcu,tcv);
 							x=liqcliprect_drawtextn_color(  cr, self->font,  x,y, alline, sellength, 255-tcy,tcv,tcu);
 							alline+=sellength;
@@ -921,19 +1146,21 @@ __tz_one("fontprep");
 							x=liqcliprect_drawtext_color(  cr, self->font,  x,y, alline,    tcy,tcu,tcv);
 							alline+=sellength;
 						}
+						x=tx;
 					}
 
 					if(cursorpos>=0)
 					{
-						x=xx;
+						//x=xx;
 						char *sel=strndup(caption,cursorpos);
 						int ttt = liqfont_textwidth(self->font,sel);
-						free(sel);
+						if(sel)free(sel);
 						liqcliprect_drawboxfillcolor(cr,x+ttt-1,y,1,fh,40,40,20);
 						liqcliprect_drawboxfillcolor(cr,x+ttt  ,y,1,fh,255,40,20);
 						liqcliprect_drawboxfillcolor(cr,x+ttt+1,y,1,fh,40,40,20);
 
 					}
+					
 
 				}
 				else
@@ -1027,11 +1254,11 @@ __tz_one("borderdone");
 				{
 					// show a position indicator bar..
 					
-					float sx = self->x;
+					//float sx = self->x;
 					float sy = self->y;
-					float ww = w;
-					float sw = self->w;
-					float pw = p->w;
+					//float ww = w;
+					//float sw = self->w;
+					//float pw = p->w;
 					float hh = h;
 					float sh = self->h;
 					float ph = p->h;		
