@@ -23,6 +23,10 @@
  */
 
 
+// 20090728_001621 lcuk : set this to have a 25fps limit to framerate, otherwise runs at fastest possible
+#define LIMIT_FRAMERATE 1
+
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -38,6 +42,7 @@
 #include "liqbase.h"
 
 #include "liqcell.h"
+#include "liqcell_prop.h"
 #include "liqcell_easyrun.h"
 #include "liqcell_easypaint.h"
 #include "vgraph.h"
@@ -71,6 +76,14 @@ static void liqcellmouseeventargs_stroke_start(liqcellmouseeventargs *self,int m
 	self->mey=my;
 	self->mez=mz;
 	self->met=mt;
+	
+	self->multiok=0;	// make sure theres no multitouch enabled yet..
+	self->multisx=0;
+	self->multisy=0;
+	self->multix=0;
+	self->multiy=0;
+	self->multiw=0;
+	self->multih=0;
 
 
 	liqstroke_clear(self->stroke);
@@ -148,7 +161,7 @@ int vrect_scaletofit(vrect *self,vrect *avail,vrect *required)
 
 
 
-static float calcaspect(int captionw,int captionh,int availw,int availh)
+float calcaspect(int captionw,int captionh,int availw,int availh)
 {
 	if(captionw==0)return 0;
 	if(captionh==0)return 0;
@@ -207,14 +220,82 @@ liqcell *liqcell_easyhittest(liqcell *self,  int mx,int my,int *hitx,int *hity)
 
 
 
+static void savethumb(liqcell *cell)
+{
+	liqcell_hold(cell);
+	// 20090528_231040 lcuk : this locks up dunno why
+	// 20090624_005139 lcuk : trying it as a widget itself, to see if it was initializers at fault
+	
+	
+	char cellname[1024];
+	
+	snprintf(cellname,sizeof(cellname),"%s",cell->name);
+	
+	liqapp_ensurecleanusername(cellname);
+	
+	liqapp_log("...creating image %s",cellname);
+	liqimage *img = liqimage_newatsize(canvas.pixelwidth,canvas.pixelheight,0);
+	
+	liqapp_log("...creating cliprect");
+	
+	liqcliprect *cr = liqcliprect_newfromimage(img);
+	
+	liqapp_log("...painting cell %s",cellname);
+	liqcell_easypaint(cell,cr,0,0,canvas.pixelwidth,canvas.pixelheight);
+	
+	liqapp_log("...building filename");
 
+				char 		fmtnow[255];
+	 			liqapp_formatnow(fmtnow,255,"yyyymmdd_hhmmss");
+				char buf[FILENAME_MAX+1];
+				snprintf(buf,FILENAME_MAX,"%s/sketches/liq.%s.%s.scr.png",app.userdatapath,fmtnow,cellname  );
+
+
+
+
+	
+	liqapp_log("...saving image as '%s'",buf);
+
+				liqimage_pagesavepng(img,buf);
+	
+
+
+//01:49:32 png writing png
+//01:49:32 png cleaning up
+//01:49:32 ...releasing cr
+//01:49:32 liqcliprect free
+//01:49:32 liqimage free
+//01:49:32 liqimage pagereset
+//01:49:32 ...releasing image
+//01:49:32 ...done
+// 20090624_015023 lcuk : a bug is occuring, the liqimage instance is being freed too early
+// 20090624_015040 lcuk : that means something is releasing it within the middle of something else
+// 20090624_015052 lcuk : but did not get hold of it first
+	
+				
+	
+	liqapp_log("...releasing cr");
+	liqcliprect_release(cr);
+	
+	liqapp_log("...releasing image");
+	liqimage_release(img);
+	
+	liqapp_log("...done");
+	
+	liqcell_release(cell);
+	
+}
+
+
+
+/*
 	static int toolitem_click(liqcell *self, liqcellclickeventargs *args, liqcell *tool)
 	{
 		//args->newdialogtoopen = liqcell_getcontent( self );
 		liqcell_setvisible(tool,0);
 		return 1;
 	}
-
+ */
 	static int tool_help_click(liqcell *self, liqcellclickeventargs *args, liqcell *tool)
 	{
 		return 1;
@@ -228,15 +309,15 @@ liqcell *liqcell_easyhittest(liqcell *self,  int mx,int my,int *hitx,int *hity)
 		return 1;
 	}
 	
-	static int tool_design_click(liqcell *self, liqcellclickeventargs *args, liqcell *tool)
+	static int tool_pic_click(liqcell *self, liqcellclickeventargs *args, liqcell *tool)
 	{
-		
 		liqcell * content = liqcell_child_lookup(tool,"content");
-		
-		int res = liqdialog_showtree("quick view","view of cell contents","",liqcell_getcontent(content) );
-		
+		savethumb( liqcell_getcontent(content) );
+		liqcell_setvisible(tool,0);
+		//int res = liqdialog_showtree("quick view","view of cell contents","",liqcell_getcontent(content) );
 		return 1;
 	}
+	
 	static int tool_pin_click(liqcell *self, liqcellclickeventargs *args, liqcell *tool)
 	{
 		return 1;
@@ -267,8 +348,8 @@ liqcell * toolclick(liqcell *vis)
 		//liqcell_propseti(  icon, "textalign", 2 );
 		liqcell_child_append(  self, icon);
 		
-		//############################# title:label
-		liqcell *title = liqcell_quickcreatevis("title", "label", 66, 8, 728, 40);
+		//############################# title:titlebar
+		liqcell *title = liqcell_quickcreatevis("title", "titlebar", 66, 8, 728, 40);
 		liqcell_setfont(	title, liqfont_cache_getttf("/usr/share/fonts/nokia/nosnb.ttf", (34), 0) );
 		liqcell_setcaption(title, liqcell_getcaption(vis) );
 		liqcell_propsets(  title, "textcolor", "rgb(255,255,255)" );
@@ -283,54 +364,86 @@ liqcell * toolclick(liqcell *vis)
 		
 		int hh = 480-64;
 
+		b = liqcell_quickcreatevis("power","liqbattery",  800-64-56,0,   56,56 );
+		//liqcell_setfont(   b, liqfont_cache_getttf("/usr/share/fonts/nokia/nosnb.ttf", (24), 0) );
+		//liqcell_handleradd_withcontext(b,    "click",   tool_help_click, self);
+		//liqcell_propsets(  b,    "backcolor", "rgb(0,100,0)" );
+		liqcell_child_append( self, b );
+
+		b = liqcell_quickcreatevis("clock","container",  800-64-56-56-56,0,   56+56,56 );
+		//liqcell_setcontent(b, liqcell_quickcreatevis("clock","ciroclock_minutes",  0,0,0,0 ) );
+		liqcell_setcontent(b, liqcell_quickcreatevis("clock","ciroclock",  0,0,0,0 ) );
+		//liqcell_setfont(   b, liqfont_cache_getttf("/usr/share/fonts/nokia/nosnb.ttf", (24), 0) );
+		//liqcell_handleradd_withcontext(b,    "click",   tool_help_click, self);
+		//liqcell_propsets(  b,    "backcolor", "rgb(0,100,0)" );
+		liqcell_child_append( self, b );
+
+
 		b = liqcell_quickcreatevis("help","button",  800-50,64+hh*0.0,   50,+hh*0.2 );
 		liqcell_setfont(   b, liqfont_cache_getttf("/usr/share/fonts/nokia/nosnb.ttf", (24), 0) );
 		liqcell_handleradd_withcontext(b,    "click",   tool_help_click, self);
 		liqcell_propsets(  b,    "backcolor", "rgb(0,100,0)" );
-		liqcell_child_insert( self, b );
+		liqcell_child_append( self, b );
 
 
-		b = liqcell_quickcreatevis("bug","button",  800-50,64+hh*0.2,   50,hh*0.2 );
+
+		b = liqcell_quickcreatevis("draw","button",  800-50,64+hh*0.2,   50,hh*0.2 );
 		liqcell_setfont(   b, liqfont_cache_getttf("/usr/share/fonts/nokia/nosnb.ttf", (24), 0) );
 		liqcell_handleradd_withcontext(b,    "click",   tool_bug_click, self);
 		liqcell_propsets(  b,    "backcolor", "rgb(100,0,0)" );
-		liqcell_child_insert( self, b );
+		liqcell_child_append( self, b );
 	
 		
 		b = liqcell_quickcreatevis("tag","button",  800-50,64+hh*0.4,   50,hh*0.2);
 		liqcell_setfont(   b, liqfont_cache_getttf("/usr/share/fonts/nokia/nosnb.ttf", (24), 0) );
 		liqcell_handleradd_withcontext(b,    "click",   tool_tag_click, self);
 		liqcell_propsets(  b,    "backcolor", "rgb(0,100,100)" );
-		liqcell_child_insert( self, b );		
+		liqcell_child_append( self, b );		
 
 
-		b = liqcell_quickcreatevis("design","button",  800-50,64+hh*0.6,   50,hh*0.2);
+
+
+		b = liqcell_quickcreatevis("pic","button",  800-50,64+hh*0.6,   50,hh*0.2);
 		liqcell_setfont(   b, liqfont_cache_getttf("/usr/share/fonts/nokia/nosnb.ttf", (24), 0) );
-		liqcell_handleradd_withcontext(b,    "click",   tool_design_click, self);
+		liqcell_handleradd_withcontext(b,    "click",   tool_pic_click, self);
 		liqcell_propsets(  b,    "backcolor", "rgb(100,100,0)" );
-		liqcell_child_insert( self, b );
+		liqcell_child_append( self, b );
 
 
-		b = liqcell_quickcreatevis("pin","button",  800-50,64+hh*0.8,   50,hh*0.2 );
+
+
+		b = liqcell_quickcreatevis("save","button",  800-50,64+hh*0.8,   50,hh*0.2 );
 		liqcell_setfont(   b, liqfont_cache_getttf("/usr/share/fonts/nokia/nosnb.ttf", (24), 0) );
 		liqcell_handleradd_withcontext(b,    "click",   tool_pin_click, self);
 		liqcell_propsets(  b,    "backcolor", "rgb(0,0,100)" );
-		liqcell_child_insert( self, b );
+		liqcell_child_append( self, b );
+
+
+
+		liqcell *content = liqcell_getcontent(vis);
+		if(content)vis=content;
+
+		// special plan here.. lets see if it can work...
+		liqcell *preview = liqcell_child_lookup(vis,"preview");
+		if(preview)vis=preview;
 
 
 
 
-
-
-
-		liqcell *c = liqcell_quickcreatevis("content", NULL, 0,56,   800-50,480-56 );
+		liqcell *c;
+		
+		
+		c = liqcell_quickcreatevis("contentdraw", "liqtop", 0,56,   800-50,480-56 );
+		//liqcell_handleradd_withcontext( c,    "click",   toolitem_click,self);
+		liqcell_child_insert( self, c );
+	
+		c = liqcell_quickcreatevis("content", NULL, 0,56,   800-50,480-56 );
 		liqcell_setcontent( c, vis );
 		liqcell_setenabled( c, 0 );		// make sure it has fadeout
-		liqcell_handleradd_withcontext( c,    "click",   toolitem_click,self);
+		//liqcell_handleradd_withcontext( c,    "click",   toolitem_click,self);
 		liqcell_child_insert( self, c );
 
 
-
 	}
 	//liqcell_easyrun(self);
 	return self;
@@ -375,7 +488,7 @@ liqcell * toolclick(liqcell *vis)
 
 
 
-
+liqcell *liqcell_findfirsthandler(liqcell*root,char *handlername);
 
 
 
@@ -383,47 +496,102 @@ liqcell * toolclick(liqcell *vis)
 
 liqcell *liqcell_findnexthandler(liqcell*self, liqcell*root,char *handlername)
 {
-	// 20090520_011358 lcuk : todo, fix this 
-		liqcell *c=liqcell_getlinkchild(self);
-		while(c && liqcell_getvisible(c)  )
+	// 20090520_011358 lcuk : todo, fix this
+	// 20090812_003812 lcuk : simplified to use _visual branch walking
+	// 20090812_010856 lcuk : complexified lol
+	
+	
+		// we have done everything with self itself
+
+		// check inside children
+		liqcell *c=liqcell_getlinkchild_visual(self);
+		while(c  )
 		//if(c && liqcell_getvisible(c)  )
 		{
 			if(liqcell_handlerfind(c,handlername))
 			{
 				return c;
 			}
-			liqcell *d = liqcell_findnexthandler(c,root,handlername);
+			liqcell *d = liqcell_findnexthandler(c,c,handlername);
 			if(d) return d;
-			c=liqcell_getlinknext(c);
+			c=liqcell_getlinknext_visual(c);
 		}
-		
-		if(self==root)return NULL;
-		//if(liqcell_getlinknext(self))
-		//	return liqcell_getlinknext(self);
-		
-		return NULL;//liqcell_getlinkparent(self);
 
+nextpar:
+		if(self==root)return NULL;
+		
+		// check all neighbours
+		liqcell *n = liqcell_getlinknext_visual(self);
+		while(n)
+		{
+			if(liqcell_handlerfind(n,handlername))
+			{
+				return n;
+			}
+			liqcell *nn = liqcell_findnexthandler(n,n,handlername);
+			if(nn)return nn;
+			n=liqcell_getlinknext_visual(n);
+		}
+
+
+
+		// at the end of the chain, only direction is backwards
+		// look at the parent, even give it a chance to be the correct handler
+		liqcell *p = liqcell_getlinkparent(self);
+		
+		if(p)
+		{
+			if(liqcell_handlerfind(p,handlername))
+			{
+				// just try this
+				return p;
+			}
+			self=p;
+			goto nextpar;
+		}
+		return NULL;
+		
+		while(p)
+		{
+
+			if(p==root)return NULL;
+			
+			liqcell *n = liqcell_getlinknext_visual(p);
+			while(n)
+			{
+				if(liqcell_handlerfind(n,handlername))
+				{
+					return n;
+				}
+				liqcell *nn = liqcell_findnexthandler(n,n,handlername);
+				if(nn)return nn;
+				n=liqcell_getlinknext_visual(n);				
+			}
+			
+			p=liqcell_getlinkparent(p);
+		}
+		return NULL;
 }
 
 liqcell *liqcell_findfirsthandler(liqcell*root,char *handlername)
 {
-			if(liqcell_handlerfind(root,handlername))
-			{
-				return root;
-			}
+		if(liqcell_handlerfind(root,handlername))
+		{
+			return root;
+		}
 
 	// 20090520_011358 lcuk : todo, fix this 
-		liqcell *c=liqcell_getlinkchild(root);
-		while(c && liqcell_getvisible(c)  )
+		liqcell *c=liqcell_getlinkchild_visual(root);
+		while(c  )
 		//if(c && liqcell_getvisible(c)  )
 		{
 			if(liqcell_handlerfind(c,handlername))
 			{
 				return c;
 			}
-			liqcell *d = liqcell_findnexthandler(c,root,handlername);
+			liqcell *d = liqcell_findnexthandler(c,c,handlername);
 			if(d) return d;
-			c=liqcell_getlinknext(c);
+			c=liqcell_getlinknext_visual(c);
 		}
 		return NULL;
 
@@ -437,6 +605,13 @@ liqcell *liqcell_findfirsthandler(liqcell*root,char *handlername)
 
 
 
+// 20090702_192535 lcuk : very quickly, the zoom in the middle of this needs replacing
+// 20090702_192547 lcuk : it needs to have an enter from parameter rect
+// 20090702_192602 lcuk : this should indicate in screen coordinates where the item is to start from
+// 20090702_192626 lcuk : during the course of the next Nth of a second the cell spends coming to rest from that point
+// 20090702_192721 lcuk : after completing, it should deblur in a similar manner.
+// 20090702_192752 lcuk : a disolve effect would be magical tonight..
+// 20090702_192827 lcuk : this completely solves the problem of the zoom in the core of this function
 
 
 
@@ -469,6 +644,10 @@ int liqcell_easyrun(liqcell *self)
 		return -1;
 	}
 	
+	// set the 
+	//liqcell_propseti(self,"dialog_complete",0);
+	liqcell_propseti(self,"dialog_running",1);
+	
 	liqcell_easyrun_depth++;
 	
 	// what i should do is resize the contents to match the frame
@@ -487,6 +666,28 @@ int liqcell_easyrun(liqcell *self)
 liqimage    *targetsurface = NULL;//liqcanvas_getsurface();
 liqcliprect *targetcr      = NULL;//liqcanvas_getcliprect();
 
+int idle_lazyrun_wanted = liqcell_propgeti(self,"idle_lazyrun_wanted",0);
+
+	int idle_lazyrun_shown(liqcell *start)
+	{
+		if(start->kineticx || start->kineticy) return 1;
+		if(liqcell_getshown(start)==0)
+		{
+			// not yet shown!
+			liqcell_handlerrun(start,"shown",NULL);
+			liqcell_setshown(start,1);
+			return 1;
+		}
+		liqcell *c=liqcell_getlinkchild_visual(start);
+		while(c)
+		{
+			if(idle_lazyrun_shown(c)) return 1;
+			c=liqcell_getlinknext_visual(c);
+		}
+		c=liqcell_getcontent(start);
+		if(c) return idle_lazyrun_shown(c);
+		return 0;
+	}
 
 
 
@@ -514,7 +715,7 @@ liqcliprect *targetcr      = NULL;//liqcanvas_getcliprect();
 int 			running=1;
 int 			result=0;
 
-unsigned long 	tzs=liqapp_GetTicks();
+unsigned long 	tzs=liqapp_GetTicks();		// technically only used when showing frameinfo, so ignore the warning :)
 unsigned long 	tz0=liqapp_GetTicks();
 unsigned long 	tz1=liqapp_GetTicks();
 LIQEVENT 		ev;
@@ -527,6 +728,10 @@ unsigned long 	refreshstarttime=0;		// if we have a refresh in progress
 
 
 
+			unsigned long 	ft1=tz0;
+			unsigned long 	ft2=tz0;
+
+
 
 liqcellpainteventargs paintargs;
 	//paintargs.cr = liqcanvas_getcliprect();
@@ -622,7 +827,7 @@ liqcell *keyhit=liqcell_findfirsthandler(self,"keypress");
 
 
 liqcell *jumpprev=NULL;
-liqcell *jumpnext=NULL;
+//liqcell *jumpnext=NULL;
 
 	liqcell *rr=liqcell_getlinkprev(self);
 	while(rr)
@@ -645,7 +850,7 @@ liqcell *jumpnext=NULL;
 		rr=liqcell_getlinknext(rr);
 	}
 	
-int jumpdir=0;		// -1== to prev,  1=tonext
+//int jumpdir=0;		// -1== to prev,  1=tonext
 
 	while(running==1)
 	{
@@ -655,7 +860,41 @@ int jumpdir=0;		// -1== to prev,  1=tonext
 		{
 waitevent:
 			//goto skipev;
+			
 
+			if(liqcanvas_eventcount()==0)
+			{
+				if(idle_lazyrun_wanted)
+				{
+					//liqapp_log("lazyrun ");
+					if( idle_lazyrun_shown(self) )
+					{
+						
+						if( idle_lazyrun_shown(self) )
+						{
+							// 2 for the price of 1 ;)
+							// we forced something on screen
+							//liqapp_log("lazyrun moar!");
+							self->dirty=1;
+						}
+						else
+						{
+							// couldn't do the second, but we managed the last one :)
+							self->dirty=1;
+							idle_lazyrun_wanted=0;
+						}
+						
+					}
+					else
+					{
+						//liqapp_log("lazyrun fin");
+						// nothing left to render :)
+						idle_lazyrun_wanted=0;
+					}
+				}
+			}
+
+			
 
 			//if(self->dirty) liqcell_setdirty(self,0);
 			//if(universe->dirty) liqcell_setdirty(universe,0);
@@ -701,18 +940,47 @@ waitevent:
 				// failing that we should open up a search box...
 				if(!keyhit)
 				{
-					// search for a cell with a key handler
+					liqapp_log("search for a cell with a key handler");
+					keyhit=liqcell_findfirsthandler(self,"keypress");
+				}
+				else
+				{
+					liqapp_log("got keyhit");
+					if(liqcell_handlerfind(keyhit,"keypress"))
+					{
+						liqapp_log("keyhit still ok : %s",keyhit->name);
+					}
+					else
+					{
+						liqapp_log("looking for new keyhit");
+						
+						liqcell *kh=keyhit=liqcell_findnexthandler(keyhit,self,"keypress");
+						if(kh && kh!=keyhit)keyhit=kh;
+						if(!kh)
+						{
+							liqapp_log("still looking");
+							kh=liqcell_findfirsthandler(self,"keypress");
+							if(kh)
+							{
+								liqapp_log("got keyhit! : %s",kh->name);
+								keyhit=kh;
+							}
+							liqapp_log("didnt find");
+						}
+						liqapp_log("dont looking");
+					}
 				}
 				if(keyhit)
 				{
 					// depending upon the key we should handle it...
-					//liqapp_log("hello?");
+					liqapp_log("hello in keyhit:: : %s",keyhit->name);
 
 					keyargs.keycode = ev.key.keycode;
 
 					strncpy(keyargs.keystring,ev.key.keystring, 16 );
 
 					keyargs.ispress=(ev.state==LIQEVENT_STATE_PRESS);
+					keyargs.keymodifierstate=ev.key.keymodifierstate;
 
 					if(ev.state==LIQEVENT_STATE_PRESS)
 					{
@@ -918,53 +1186,54 @@ waitevent:
 				{
 					// still using the right area :)
 
-					//if( liqcell_handlerrun(hot,"mouse",&mouseargs) )
-					//{
-					//}
-						//liqapp_log("mouse test '%s'",hot->name);
-
+					{
+						int vx=hotx;
+						int vy=hoty;
+						liqcell *vhit=hot;
+						while(vhit && (liqcell_handlerfind(vhit,"mouse")==NULL)  )
 						{
-							int vx=hotx;
-							int vy=hoty;
-							liqcell *vhit=hot;
-							while(vhit && (liqcell_handlerfind(vhit,"mouse")==NULL)  )
-							{
-								//liqapp_log("mouse skip  '%s'",vhit->name);
+							//liqapp_log("mouse skip  '%s'",vhit->name);
 
-								vx-=vhit->x;
-								vy-=vhit->y;
-								vhit=liqcell_getlinkparent(vhit);
+							vx-=vhit->x;
+							vy-=vhit->y;
+							vhit=liqcell_getlinkparent(vhit);
+						}
+						if(vhit)
+						{
+							//liqapp_log("mouse run  '%s'",vhit->name);
+							
+							
+							
+
+							//##########################################
+							// get absolute offset (make this a cell_fn?) must stop when it gets to the dialog item itself though
+							int ox=0;
+							int oy=0;
+							liqcell *ohit=vhit;
+							while(ohit && ohit!=self)
+							{
+								ox+=ohit->x;
+								oy+=ohit->y;
+								ohit=liqcell_getlinkparent(ohit);
 							}
-							if(vhit)
+
+							mouseargs.ox=ox;
+							mouseargs.oy=oy;
+							
+							
+							liqcell_easyrun_mouseeventargs_multitouchprepare(vhit,&mouseargs,NULL);
+
+							if( liqcell_handlerrun(vhit,"mouse",&mouseargs) )
 							{
-								//liqapp_log("mouse run  '%s'",vhit->name);
-
-								//##########################################
-								// get absolute offset (make this a cell_fn?) must stop when it gets to the dialog item itself though
-								int ox=0;
-								int oy=0;
-								liqcell *ohit=vhit;
-								while(ohit && ohit!=self)
+								// handled it \o/
+								if(self->visible)
 								{
-									ox+=ohit->x;
-									oy+=ohit->y;
-									ohit=liqcell_getlinkparent(ohit);
-								}
-
-								mouseargs.ox=ox;
-								mouseargs.oy=oy;
-
-								if( liqcell_handlerrun(vhit,"mouse",&mouseargs) )
-								{
-									// handled it \o/
-									if(self->visible)
-									{
-										// refresh display
-										// sleep
-									}
+									// refresh display
+									// sleep
 								}
 							}
 						}
+					}
 
 
 					if(ev.mouse.pressure==0)
@@ -1051,7 +1320,7 @@ quickfin:
 
 			else if(ev.type == LIQEVENT_TYPE_DIRTYFLAGSET && (refreshinprogress==0))
 			{
-				liqapp_log("event dirty");
+			//	liqapp_log("event dirty");
 				
 				// ok, we want to be exposed
 				//refreshinprogress=0;
@@ -1112,17 +1381,18 @@ quickfin:
 			}
 		}
 
-skipev:
+//skipev:
 
 
 		if(refreshinprogress==0 && self->visible==0)break;
+		
+		// check for dialog complete dignal and bail
+		
+		if(refreshinprogress==0 && liqcell_propgeti(self,"dialog_running",1)==0 ) break;
 
 
 
 		float zoom_duration = 0.15;//0.4;//0.01;//0.1;//0.2;	// time to go from fullscreen to zoomed in
-		
-		//zoom_duration=2;	// 20090410_130608 lcuk : testing the layout stuff
-
 		if((zoom_in_progress) && (hot) && (refreshinprogress==0))
 		{
 			float zoomruntime = (liqapp_GetTicks()-zoom_start) / (1000.0);
@@ -1156,30 +1426,17 @@ skipev:
 			}
 			else
 			{
-
 moar:
-
 				if(zoom_direction==-1) zoomfactor = 1-zoomfactor;
-
-
 				//int hisa = calcaspect(hot->w,hot->h, self->w,self->h);
 				//int hisw = hot->w * hisa;
 				//int hish = hot->h * hisa;
 				float hfx=1;//(float)hisw / (float)self->w;
 				float hfy=1;//(float)hish / (float)self->h;
-
-
 				// we are some fraction between viewing the whole screen and being zoomed directly on hot
 				//
-
 				zw = self->w + (float)self->w * zoomfactor * (((float)self->w / (float)hot->w * hfx)-1 );
 				zh = self->h + (float)self->h * zoomfactor * (((float)self->h / (float)hot->h * hfy)-1 );
-
-
-				//zw = self->w + (float)hisw * zoomfactor * (((float)hisw / (float)hot->w)-1);
-				//zh = self->h + (float)hish * zoomfactor * (((float)hish / (float)hot->h)-1);
-
-
 				int rx=0;
 				int ry=0;
 				liqcell *r=hot;
@@ -1190,130 +1447,59 @@ moar:
 					ry+=r->y;
 					r=r->linkparent;
 				}
-				//rx+=hot->w/2;
-				//ry+=hot->h/2;
-
-
 				zx = -(float)rx * (zoomfactor) * (((float)self->w / (float)hot->w * hfx));
 				zy = -(float)ry * (zoomfactor) * (((float)self->h / (float)hot->h * hfy));
 
-
-				//zx = -(float)rx * (zoomfactor) * (((float)hisw / (float)hot->w));
-				//zy = -(float)ry * (zoomfactor) * (((float)hish / (float)hot->h));
-
-
-
-
 				//liqapp_log("self(%i,%i)   hot(%i,%i)-step(%i,%i)   z(%i,%i)-step(%i,%i)",
 				//					self->w,self->h,
 				//					hot->x,hot->y, hot->w,hot->h,
 				//					zx,zy,zw,zh);
-
-
 			}
 
 			dirty=1;
 		}
-
-
-
-		//if(self->dirty && (refreshinprogress==0))
-		//if(universe->dirty && (refreshinprogress==0) && (dirty==0))
-		//{
-		//	dirty=1;
-		//	universe->dirty=0;
-		//}
-
 		if(self->dirty && (refreshinprogress==0))// && (dirty==0))
 		{
 			dirty=1;
 			self->dirty=0;
 		}
-
-
-
-
-		//if((refreshinprogress==0) )//&& (wantwait==0))
-		//{
-		//	dirty=1;
-		//}
-
-
 		if(refreshinprogress==0) if(running==0) break;
-
-
 		if(paintargs.runfast) dirty=1;
-
-
-		//if(self->dirty && (refreshinprogress==0))
-		//if(universe->dirty && (refreshinprogress==0) && (dirty==0))
-		//{
-			// leave the system telling us its changed until we have finished trying all normal updates..
-		//	dirty=1;
-			//universe->dirty=0;
-		//}
-
-
-
-		//dirty=1;
-		//wantwait=0;
-		//refreshinprogress=0;
-
-
 		if(((dirty==1) && (refreshinprogress==0)))
 		{
-			
-
 		//	liqapp_log("render %i  ud=%i",framecount,universe->dirty);
-
 			//liqapp_log("rendering %i",framecount);
-
 			//liqcliprect_drawclear(liqcanvas_getcliprect(),255,128,128);
 			liqcliprect_drawclear(liqcanvas_getcliprect(),0,128,128);
-
 			// ensure runfast is unset before attmpting the next loop
 			paintargs.runfast=0;
-
 			//##################################################### render handler
 			// do whatever we want..
-			liqcell_handlerrun(self,"paint",&paintargs);
-
+			
+			// 20090812_204110 lcuk : wow - i have been calling the paint event twice all the time
+			// 20090812_204123 lcuk : lots of things dont need it, but this does. very interesting.
+			
+			//liqcell_handlerrun(self,"paint",&paintargs);
 			float fac = 1;
-
 			fac=1;
-
 			int w=(((float)self->w)*fac);
 			int h=(((float)self->h)*fac);
 			int x=0;//self->x;//-w/2;
 			int y=0;//self->y;//-h/2;
-
-
 			if(zoom_in_progress)
 			{
 				w=zw;
 				h=zh;
 				x=zx;
 				y=zy;
-
 			}
-			
-
-
 			if(mouseargs.mcnt>0)
 			{
 			//	w=ev.mouse.x;
 			}
-
-
 			//liqapp_log("render drawing wh(%i,%i)",w,h);
-
-
 			vgraph_drawcell(graph,x,y,w,h,self);
-			
-			
 			//liqapp_log("render adding nav items");
-
-
 			static liqimage *infoback=NULL;
 			static liqimage *infoclose=NULL;
 			static liqimage *infotools=NULL;
@@ -1321,17 +1507,17 @@ moar:
 			if(!infoback)
 			{
 				//liqapp_log("************************************************************************************** read");
-				infoback = liqimage_cache_getfile("/usr/share/liqbase/media/quickicons/back.png", 0,0,1);
+				infoback = liqimage_cache_getfile("/usr/share/liqbase/libliqbase/media/back.png", 0,0,1);
 			}
 			if(!infoclose)
 			{
 				//liqapp_log("************************************************************************************** read");
-				infoclose = liqimage_cache_getfile("/usr/share/liqbase/media/quickicons/gtk-close.png", 0,0,1);
+				infoclose = liqimage_cache_getfile("/usr/share/liqbase/libliqbase/media/gtk-close.png", 0,0,1);
 			}
 			if(!infotools)
 			{
 				//liqapp_log("************************************************************************************** read");
-				infotools = liqimage_cache_getfile("/usr/share/liqbase/media/quickicons/package_system.png", 0,0,1);
+				infotools = liqimage_cache_getfile("/usr/share/liqbase/libliqbase/media/package_system.png", 0,0,1);
 			}
 			if(infoback && infoclose)
 			{
@@ -1344,17 +1530,22 @@ moar:
 			}
 			if( infotools )
 			{
-
-
 					liqcliprect_drawimagecolor(targetcr, infotools , targetsurface->width-48,0 ,48,48, 1);
-
 			}
  		
-			
-			//liqapp_log("render adding framecount");
-			
+			// 20090712_224448 lcuk : i want MAX 25fps, just to see
+			//ft2=liqapp_GetTicks();
+#ifdef LIMIT_FRAMERATE		
+			while( ( (ft2=liqapp_GetTicks()) - ft1) < 40)
+			{
+				// lets just sleep for a short while (doesnt have to be THAT precise, just stop silly speed overruns)
+				//liqapp_log("sleeping %i",100 - (ft2-ft1));
+				liqapp_sleep(40 - (ft2-ft1));
+			}
+			ft1=ft2;
+#endif					
+			//liqapp_log("render adding framecount");		
 // 20090520_014021 lcuk : show frame information
-
 /*
 			static liqfont *infofont=NULL;
 			if(!infofont)
@@ -1365,36 +1556,33 @@ moar:
 			{
 				char *cap=liqcell_getcaption(self);
 				if(!cap || !*cap) cap="[nameless]";
-				
 				char buff[255];
 				snprintf(buff,sizeof(buff),"liqbase '%s' %3i, %3.3f, %3.3f",cap,framecount, liqapp_fps(tz0,tz1,1) ,liqapp_fps(tzs,tz1,framecount) );
-				
-				
 				//liqapp_log(buff);
-
-
 				int hh=liqfont_textheight(infofont);
 				liqcliprect_drawtextinside_color(targetcr, infofont,  0,0, targetsurface->width,hh, buff,0, 255,128,128);
-
 			}
-			
-			
- */		
-			
-			
-
- 
+ */			
+ 		
 			//liqapp_log("render refreshing");
-
-			liqcanvas_refreshdisplay();
 			
-			//liqapp_log("render done");
+			
 
+			
+			
+			liqcanvas_refreshdisplay();
+
+
+
+
+
+
+
+
+			//liqapp_log("render done");
 			framecount++;
 			dirty=0;
 
-
-
 // Sun Apr 05 16:49:47 2009 lcuk : kots x86 machine does not send back refresh events
 // needs this for now until we work out why
 //#ifndef USE_MAEMO
@@ -1430,15 +1618,10 @@ moar:
 		}
 	}
 	liqapp_log("liqcell easyrun complete %i",result);
-
 	vgraph_release(graph);
-
 	liqstroke_release(mouseargs.stroke);
-
-
 	liqcell_handlerrun(self,"dialog_close",NULL);
-
+	liqcell_propremovei(self,"dialog_running");
 	liqcell_easyrun_depth--;
-
 	return result;
 }
